package es.verdnatura.db

import android.content.Context
import androidx.room.*


// Clase que representa la base de datos
@Database(entities = [Expedicion::class], version = 1)
abstract class ExpedicionDatabase : RoomDatabase() {
    abstract fun expedicionDao(): ExpedicionDao

    companion object {
        @Volatile
        private var INSTANCE: ExpedicionDatabase? = null

        fun getInstance(context: Context): ExpedicionDatabase {
            return INSTANCE ?: synchronized(this) {
                val instance = Room.databaseBuilder(
                    context.applicationContext,
                    ExpedicionDatabase::class.java,
                    "expedicion_database"
                )
                    .fallbackToDestructiveMigration()
                    .build()
                INSTANCE = instance
                instance
            }
        }
    }
}

// Clase que representa la entidad Expedicion
@Entity(tableName = "expediciones")
data class Expedicion(
    @PrimaryKey val id: Int,
    val nombre: String,
    val fecha: Long
)

// Interfaz que define los métodos de acceso a datos para Expedicion
@Dao
interface ExpedicionDao {
    @Query("SELECT * FROM expediciones")
    suspend fun getAll(): List<Expedicion>

    @Insert(onConflict = OnConflictStrategy.REPLACE)
    suspend fun insert(expedicion: Expedicion)

    @Delete
    fun delete(expedicion: Expedicion)
}

