package es.verdnatura.presentation.view.feature.ajustes.fragment


import android.content.Context
import android.view.View.GONE
import android.view.View.VISIBLE
import androidx.lifecycle.LiveData
import androidx.lifecycle.MutableLiveData
import androidx.lifecycle.Transformations
import es.verdnatura.R
import es.verdnatura.domain.SilexCallback
import es.verdnatura.presentation.base.BaseViewModel
import es.verdnatura.presentation.base.getMessageFromAllResponse
import es.verdnatura.presentation.base.nameofFunction
import es.verdnatura.presentation.common.Event
import es.verdnatura.presentation.common.ResponseItemNumber
import es.verdnatura.presentation.common.ResponseItemVO
import es.verdnatura.presentation.view.feature.ajustes.model.*
import es.verdnatura.presentation.view.feature.articulo.model.ItemPackingType
import es.verdnatura.presentation.view.feature.articulo.model.ItemPackingTypeList
import retrofit2.Response


class AjustesViewModel(val context: Context) : BaseViewModel(context) {

    private val _ajustesitem by lazy { ArrayList<AjustesItemVO>() }
    val ajustesitem: List<AjustesItemVO>
        get() = _ajustesitem

    private val _sectorList by lazy { MutableLiveData<SectorListVO>() }
    val sectorList: LiveData<SectorListVO>
        get() = _sectorList

    private val _actionUpdate by lazy { MutableLiveData<ResponseItemVO>() }
    val actionUpdate: LiveData<ResponseItemVO>
        get() = _actionUpdate

    private val _actionUpdateTrain by lazy { MutableLiveData<ResponseItemVO>() }
    val actionUpdateTrain: LiveData<ResponseItemVO>
        get() = _actionUpdateTrain


    private val _actionUpdatePacking by lazy { MutableLiveData<ResponseItemVO>() }
    val actionUpdatePacking: LiveData<ResponseItemVO>
        get() = _actionUpdatePacking

    private val _actionUpdatePackingTicket by lazy { MutableLiveData<ResponseItemVO>() }
    val actionUpdatePackingTicket: LiveData<ResponseItemVO>
        get() = _actionUpdatePackingTicket

    private val _workerupdatePrinter by lazy { MutableLiveData<ResponseItemVO>() }
    val workerupdatePrinter: LiveData<ResponseItemVO>
        get() = _workerupdatePrinter

    private val _printerList by lazy { MutableLiveData<PrintersList>() }
    val printerList: LiveData<PrintersList>
        get() = _printerList

    private val _operatorGetNumberOfWagons by lazy { MutableLiveData<ResponseItemNumber>() }
    val operatorGetNumberOfWagons: LiveData<ResponseItemNumber>
        get() = _operatorGetNumberOfWagons

    private val _operatorGetTrainResponse by lazy { MutableLiveData<ResponseItemVO>() }
    val operatorGetTrainResponse: LiveData<ResponseItemVO>
        get() = _operatorGetTrainResponse

    private val _operatorGetItemPackingType by lazy { MutableLiveData<ResponseItemVO>() }
    val operatorGetItemPackingType: LiveData<ResponseItemVO>
        get() = _operatorGetItemPackingType

    private val _workerAllowedResponse by lazy { MutableLiveData<ResponseItemVO>() }
    val workerAllowedResponse: LiveData<ResponseItemVO>
        get() = _workerAllowedResponse

    private val _itemspackinglist by lazy { MutableLiveData<ItemPackingTypeList>() }
    val itemspackinglist: LiveData<ItemPackingTypeList>
        get() = _itemspackinglist

    private val _trainList by lazy { MutableLiveData<TrainList>() }
    val trainList: LiveData<TrainList>
        get() = _trainList

    val loadSectorList = Transformations.map(_sectorList) { Event(it) }
    val loadPrintersList = Transformations.map(_printerList) { Event(it) }
    val loadItemPackingList = Transformations.map(_itemspackinglist) { Event(it) }
    val loadTrainList = Transformations.map(_trainList) { Event(it) }
    val loadUpdatePacking = Transformations.map(_actionUpdatePackingTicket) { Event(it) }

    private val _isLoadingProgress by lazy { MutableLiveData<Int>() }
    val isLoadingProgress: LiveData<Int>
        get() = _isLoadingProgress

    fun ProgressLoading(visibility: Int) {
        _isLoadingProgress.value = visibility
    }


    fun inititializeDefaultAjusts(
        sectorDescrip: String,
        sectorFk: Int,
        warehouseFk: Int,
        carroDescrip: String,
        printerFk: Int,
        printerSelected: String,
        train: String?,
        itemPackingType: String?,
        actionIsAllowed: Boolean
    ) {
        _ajustesitem.clear()
        _ajustesitem.add(
            AjustesItemVO(
                0,
                context.getString(R.string.sector),
                sectorDescrip,
                sectorFk,
                warehouseFk,
                0,
                printerSelected,
                true
            )
        )


        _ajustesitem.add(
            AjustesItemVO(
                1,
                context.getString(R.string.wagoons),
                carroDescrip,
                0,
                0,
                0, context.getString(R.string.printerWithout),
                false
            )
        )
        _ajustesitem.add(
            AjustesItemVO(
                2,
                context.getString(R.string.printerLabel),
                printerSelected,
                0,
                0,
                printerFk,
                printerSelected,
                true
            )
        )
        _ajustesitem.add(
            AjustesItemVO(
                6,
                context.getString(R.string.settingsTitleTrain),
                train,
                0,
                0,
                0,
                action = actionIsAllowed
            )
        )
        _ajustesitem.add(
            AjustesItemVO(
                7,
                context.getString(R.string.settingsTitleItemPacking),
                itemPackingType,
                0,
                0,
                0,
                action = actionIsAllowed
            )
        )

        _ajustesitem.add(
            AjustesItemVO(
                4,
                context.getString(R.string.vehicleControl),
                "",
                0,
                0,
                0, context.getString(R.string.printerWithout),
                true
            )
        )
        _ajustesitem.add(
            AjustesItemVO(
                5,
                context.getString(R.string.closeSession),
                "",
                0,
                0,
                0, context.getString(R.string.printerWithout),
                true
            )
        )

    }


    fun getItemPackingTypeSalix() {
        salix.get_ItemPackingType().enqueue(object :
            SilexCallback<List<ItemPackingType>>(context) {
            override fun onError(t: Throwable) {

                val listError: ArrayList<ItemPackingType> = ArrayList()
                listError.add(
                    ItemPackingType(
                        isError = true,
                        errorMessage = getMessageFromAllResponse(nameofFunction(this), t.message!!)
                    )
                )
                _itemspackinglist.value = ItemPackingTypeList(listError)
            }

            override fun onSuccess(response: Response<List<ItemPackingType>>) {

                if (response.body() != null) {
                    _itemspackinglist.value = response.body()?.let { ItemPackingTypeList(it) }

                } else {
                    val listError: ArrayList<ItemPackingType> = ArrayList()
                    listError.add(
                        ItemPackingType(
                            isError = true,
                            errorMessage = getMessageFromAllResponse(
                                nameofFunction(this),
                                response.message()
                            )
                        )
                    )
                    _itemspackinglist.value = ItemPackingTypeList(listError)
                }
            }

        })
    }

    fun getSectorsSalix() {
        salix.getSectorsSalix().enqueue(object : SilexCallback<List<SectorItemVO>>(context) {
            override fun onError(t: Throwable) {

                val listError: ArrayList<SectorItemVO> = ArrayList()
                listError.add(SectorItemVO(0, "", 0, true, t.message!!))
                _sectorList.value = SectorListVO(listError)
            }

            override fun onSuccess(response: Response<List<SectorItemVO>>) {

                if (response.body() != null) {
                    _sectorList.value = response.body()?.let { SectorListVO(it) }
                } else {
                    val listError: ArrayList<SectorItemVO> = ArrayList()
                    listError.add(
                        SectorItemVO(
                            0, "", 0, true,
                            getMessageFromAllResponse(nameofFunction(this), response.message())
                        )
                    )
                    _sectorList.value = SectorListVO(listError)
                }
            }

        })
    }

    fun printer_get() {
        silex.printer_get().enqueue(object : SilexCallback<List<Printers>>(context) {
            override fun onError(t: Throwable) {

                val listError: ArrayList<Printers> = ArrayList()
                listError.add(Printers(0, "", true, t.message!!))
                _printerList.value = PrintersList(listError)

            }

            override fun onSuccess(response: Response<List<Printers>>) {

                if (response.body() != null) {
                    _printerList.value = response.body()?.let { PrintersList(it) }
                } else {
                    val listError: ArrayList<Printers> = ArrayList()
                    listError.add(
                        Printers(
                            0, "", true,
                            getMessageFromAllResponse(nameofFunction(this), response.message())
                        )
                    )
                    _printerList.value = PrintersList(listError)
                }
            }

        })
    }

    fun train_get() {
        silex.train_get().enqueue(object : SilexCallback<List<Train>>(context) {
            override fun onError(t: Throwable) {

                val listError: ArrayList<Train> = ArrayList()
                listError.add(Train(0, "", true, t.message!!))
                _trainList.value = TrainList(listError)

            }

            override fun onSuccess(response: Response<List<Train>>) {

                if (response.body() != null) {
                    _trainList.value = response.body()?.let { TrainList(it) }
                } else {
                    val listError: ArrayList<Train> = ArrayList()
                    listError.add(
                        Train(
                            0, "", true,
                            getMessageFromAllResponse(nameofFunction(this), response.message())
                        )
                    )
                    _trainList.value = TrainList(listError)
                }
            }

        })
    }

    fun worker_updateSector(sectorFk: Int) {

        silex.worker_updateSector(sectorFk)
            .enqueue(object : SilexCallback<Unit>(context) {
                override fun onError(t: Throwable) {

                    _actionUpdate.value = ResponseItemVO(
                        isError = true,
                        errorMessage = getMessageFromAllResponse(nameofFunction(this), t.message!!)
                    )
                }

                override fun onSuccess(response: Response<Unit>) {
                    _actionUpdate.value = ResponseItemVO(
                        response = response.message(),
                        isError = false,
                        errorMessage = ""

                    )

                }

            })
    }

    fun operator_updateItemPackingType(itemPackingTypeFk: String) {

        silex.operator_updateItemPackingType(itemPackingTypeFk)
            .enqueue(object : SilexCallback<Unit>(context) {
                override fun onError(t: Throwable) {

                    _actionUpdatePacking.value = ResponseItemVO(
                        isError = true,
                        errorMessage = getMessageFromAllResponse(nameofFunction(this), t.message!!)
                    )
                }

                override fun onSuccess(response: Response<Unit>) {

                    _actionUpdatePacking.value = ResponseItemVO(
                        response = response.message(),
                        isError = false,
                        errorMessage = ""

                    )

                }

            })
    }

    fun operator_getNumberOfWagons() {

        silex.operator_getNumberOfWagons()
            .enqueue(object : SilexCallback<Int?>(context) {

                override fun onError(t: Throwable) {

                    _operatorGetNumberOfWagons.value = ResponseItemNumber(
                        null,
                        isError = true,
                        errorMessage = getMessageFromAllResponse(nameofFunction(this), t.message!!)
                    )
                }

                override fun onSuccess(response: Response<Int?>) {
                    if (response.body() != null)
                        _operatorGetNumberOfWagons.value =
                            ResponseItemNumber(response.body(), isError = false)

                }

            })
    }

    fun operator_getTrain() {

        silex.operator_getTrain()
            .enqueue(object : SilexCallback<String?>(context) {

                override fun onError(t: Throwable) {
                    _operatorGetTrainResponse.value = ResponseItemVO(
                        isError = true,
                        errorMessage = getMessageFromAllResponse(nameofFunction(this), t.message!!)
                    )
                }

                override fun onSuccess(response: Response<String?>) {
                    _operatorGetTrainResponse.value =
                        ResponseItemVO(
                            if (response.body().toString() == "false") "" else {
                                response.body().toString()
                            }, false
                        )
                }
            })
    }

    fun operator_getItemPackingType() {

        silex.operator_getItemPackingType()
            .enqueue(object : SilexCallback<Any?>(context) {
                override fun onError(t: Throwable) {
                    _operatorGetItemPackingType.value = ResponseItemVO(
                        isError = true,
                        errorMessage = getMessageFromAllResponse(nameofFunction(this), t.message!!)
                    )
                }

                override fun onSuccess(response: Response<Any?>) {

                    _operatorGetItemPackingType.value =
                        ResponseItemVO(
                            if (response.body().toString() == "false") "" else {
                                response.body().toString()
                            }, false
                        )

                }
            })
    }

    fun worker_isF11Allowed() {
        silex.worker_isF11Allowed()
            .enqueue(object : SilexCallback<Boolean>(context) {
                override fun onError(t: Throwable) {

                    _workerAllowedResponse.value = ResponseItemVO(
                        isError = true,
                        errorMessage = getMessageFromAllResponse(nameofFunction(this), t.message!!)
                    )
                }

                override fun onSuccess(response: Response<Boolean>) {
                    _workerAllowedResponse.value = ResponseItemVO(
                        response = response.body().toString(),
                        isError = false,
                    )
                }

            })
    }

    fun worker_updatePrinter(printerFk: Int) {
        ProgressLoading(VISIBLE)
        silex.worker_updatePrinter(printerFk)
            .enqueue(object : SilexCallback<Unit>(context) {
                override fun onError(t: Throwable) {
                    ProgressLoading(GONE)
                    throw t
                }

                override fun onSuccess(response: Response<Unit>) {
                    ProgressLoading(GONE)
                    super.onSuccess(response)
                }

            })
    }

    fun operator_updateTrain(trainFk: Int) {
        ProgressLoading(VISIBLE)
        silex.operator_updateTrain(trainFk)
            .enqueue(object : SilexCallback<Unit>(context) {

                override fun onError(t: Throwable) {
                    ProgressLoading(GONE)
                    throw t
                }

                override fun onSuccess(response: Response<Unit>) {
                    ProgressLoading(GONE)
                    super.onSuccess(response)
                }

            })
    }
}

