package es.verdnatura.presentation.view.feature.delivery.fragments

import android.content.Intent
import android.content.pm.PackageInfo
import android.content.pm.PackageManager
import android.graphics.drawable.Drawable
import android.os.Build
import android.view.View
import android.view.inputmethod.EditorInfo
import android.widget.ImageView
import android.widget.Toast
import androidx.activity.OnBackPressedCallback
import androidx.annotation.RequiresApi
import androidx.lifecycle.ViewModelProvider
import androidx.lifecycle.lifecycleScope
import androidx.recyclerview.widget.LinearLayoutManager
import androidx.room.Room
import es.verdnatura.R
import es.verdnatura.databinding.FragmentExpeditionLoadunloadDeliveryBinding
import es.verdnatura.db.Expedicion
import es.verdnatura.db.ExpedicionDatabase
import es.verdnatura.domain.toast
import es.verdnatura.presentation.base.BaseFragment
import es.verdnatura.presentation.common.OnOptionsSelectedListener
import es.verdnatura.presentation.common.ToolBarAdapterTooltip
import es.verdnatura.presentation.view.feature.delivery.adapters.ExpeditionLoadUnloadAdapter
import es.verdnatura.presentation.view.feature.delivery.model.ExpeditionInfoLoadUnload
import es.verdnatura.presentation.view.feature.delivery.viewmodels.DatabaseViewModel
import es.verdnatura.presentation.view.feature.delivery.viewmodels.DeliveryViewModel
import kotlinx.coroutines.Dispatchers
import kotlinx.coroutines.launch
import kotlinx.coroutines.withContext


class LoadUnloadFragment(
    var title: String = "",
    var state: String = ""
) : BaseFragment<FragmentExpeditionLoadunloadDeliveryBinding, DeliveryViewModel>(
    DeliveryViewModel::class
) {

    private var adapter: ExpeditionLoadUnloadAdapter? = null
    private var originalItem: Int = 0
    private lateinit var myList: MutableList<ExpeditionInfoLoadUnload>
    private var counterCalls = 0
    private var counterResponse = 0


    companion object {
        fun newInstance(title: String, state: String) = LoadUnloadFragment(title, state)
    }

    override fun getLayoutId(): Int = R.layout.fragment_expedition_loadunload_delivery

    @RequiresApi(Build.VERSION_CODES.O)
    override fun init() {


        setEvents()
        setToolBar()
        binding.scanInput.requestFocus()
        viewModel.getExpeditionFromRoute(159390)
        super.init()
        database()
    }

    private fun database() {

        val db = Room.databaseBuilder(
            requireContext().applicationContext,
            ExpedicionDatabase::class.java, "expediciones.db"
        ).build()

// Para insertar una expedición en la base de datos, puedes hacer lo siguiente:
/*        val expedicion = Expedicion(12345, "Mi expedición", System.currentTimeMillis())
        db.expedicionDao().insert(expedicion)*/

        // val myViewModel = ViewModelProvider(this).get(DatabaseViewModel::class.java)

        lifecycleScope.launch {
            val expedicionInsert = withContext(Dispatchers.IO) {
                val expedicion = Expedicion(12345, "Mi expedición", System.currentTimeMillis())
                db.expedicionDao().insert(expedicion)
                var expediciones = db.expedicionDao().getAll()
                for (e in expediciones){
                    println("la exp es "+e.nombre)
                }
            }
        }

// Para obtener todas las expediciones de la base de datos, puedes hacer lo siguiente:

        /* db.expedicionDao().getAll().observe(viewLifecycleOwner) { expediciones ->
             // actualizar la vista con las expediciones
         }
         val expediciones = db.expedicionDao().getAll()
         for (e in expediciones) {
             println("El nombre de la exp es " + e.nombre)
         }*/
    }

    @RequiresApi(Build.VERSION_CODES.O)
    private fun setToolBar() {
        ma.hideBottomNavigation(View.GONE)
        binding.mainToolbar.toolbarTitle.visibility = View.VISIBLE
        binding.mainToolbar.toolbarSubtitle.visibility = View.VISIBLE
        binding.mainToolbar.toolbarIcons.visibility = View.VISIBLE
        binding.mainToolbar.switchButton.visibility = if (state != "DELIVERED") {
            View.VISIBLE
        } else {
            View.GONE
        }
        binding.mainToolbar.toolbarTitle.text = title

        val listIcons: ArrayList<ImageView> = ArrayList()
        val iconConfirm = ImageView(context)
        iconConfirm.setImageResource(R.drawable.ic_confirm)
        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.O) {
            iconConfirm.tooltipText = getTooltip(R.drawable.ic_confirm)
        }
        listIcons.add(iconConfirm)

        binding.mainToolbar.toolbarIcons.adapter =
            ToolBarAdapterTooltip(listIcons, object : OnOptionsSelectedListener {

                override fun onOptionsItemSelected(item: Drawable) {

                    when (item) {
                        iconConfirm.drawable -> {
                            // openAppDelivery()
                            //requireActivity().finish()
                            setExpeditionsState(myList, state)
                        }
                    }
                }
            })

        binding.mainToolbar.toolbarIcons.layoutManager =
            LinearLayoutManager(requireContext(), LinearLayoutManager.HORIZONTAL, false)
        binding.mainToolbar.toolbarIcons.visibility = View.VISIBLE
    }

    private fun setExpeditionsState(list: MutableList<ExpeditionInfoLoadUnload>, state: String) {


        if (list.filter { it.code == state }.size == list.size) {
            binding.splashProgress.visibility = View.VISIBLE

            viewModel.expeditionState_add(list.filter { it.code == state }, state)
            counterCalls = 1
        } else {
            val alertDialog = android.app.AlertDialog.Builder(requireContext()).create()
            alertDialog.setTitle("Información")
            alertDialog.setMessage("El total de expediciones escaneadas es diferente al total de la ruta. ¿ Quieres guardar las expediciones escaneadas o buscar la/s expedición/es ?")
            alertDialog.setButton(
                android.app.AlertDialog.BUTTON_NEUTRAL, "GUARDAR"
            ) { dialog, which ->
                counterCalls = 2
                viewModel.expeditionState_add(list.filter { it.code == state }, state)
                viewModel.expeditionState_add(list.filter { it.code != state }, "LOST")
            }
            alertDialog.setButton(
                android.app.AlertDialog.BUTTON_POSITIVE, "ORDENAR"
            ) { dialog, which ->
                createList(list.sortedByDescending { it.code })
            }
            alertDialog.show()
        }


    }

    private fun openAppDelivery() {
        val i = Intent()
        val manager: PackageManager = requireContext().packageManager
        val info: PackageInfo = manager.getPackageInfo(requireContext().packageName, 0)
        val version = info.versionName

        if (version.contains("Beta")) {
            i.setClassName(
                "verdnatura.es.repartoverdnatura.sfusion",
                "verdnatura.es.repartoverdnatura.ExpeditionSummaryActivity"
            )
        } else {
            i.setClassName(
                "verdnatura.es.repartoverdnatura",
                "verdnatura.es.repartoverdnatura.ExpeditionSummaryActivity"
            )
        }
        // i.putExtra("routeId", binding.route.text)
        i.putExtra("app", "picking")
        startActivity(i)
    }

    private fun setEvents() {
        binding.mainToolbar.backButton.setOnClickListener {
            /* requireActivity().onBackPressed() */
            val callback = object : OnBackPressedCallback(true /* enabled by default */) {
                override fun handleOnBackPressed() {
                    println("volviendo antras")
                }
            }

            requireActivity().onBackPressedDispatcher.addCallback(viewLifecycleOwner, callback)
        }

        binding.scanInput.setOnEditorActionListener { _, actionId, _ ->
            if (actionId == EditorInfo.IME_ACTION_SEARCH || actionId == EditorInfo.IME_ACTION_DONE || actionId == 0) {
                if (!binding.scanInput.text.isNullOrEmpty()) {

                    try {
                        //viewModel.expeditionLoadUnload_get(binding.scanInput.toInt())
                        markExpedition(
                            binding.scanInput.text.toString().toInt(),
                            if (!binding.mainToolbar.switchButton.isChecked) {
                                state
                            } else {
                                "FOUND"
                            },

                            )
                        binding.mainToolbar.toolbarTitle.text =
                            if (!binding.mainToolbar.switchButton.isChecked) {
                                getString(R.string.loading)
                            } else {
                                getString(R.string.unloading)
                            }


                    } catch (ex: Exception) {
                        getString(R.string.errorExpedition).toast(context, Toast.LENGTH_SHORT)
                    }
                    binding.scanInput.setText("")
                }

                return@setOnEditorActionListener false
            }
            false
        }

    }

    private fun markExpedition(expedNumber: Int, state: String) {

        var found = false

        if (myList.filter { it.id == expedNumber }.filter { it.code == state }.isNotEmpty()) {
            ma.messageWithSound(getString(R.string.expeditionMarked), null, true)
        } else {
            for (exped in myList) {
                if (exped.id == expedNumber) {
                    found = true
                    exped.code = state
                    break
                }
            }
            if (found) {
                ma.messageWithSound("", false, true, isToasted = false)
            } else {
                ma.messageWithSound("", true, true, isToasted = null)
            }
        }

        updateCount(myList, state)


    }

    @RequiresApi(Build.VERSION_CODES.O)
    override fun observeViewModel() {
        with(viewModel) {
            expeditionInfoLoadUnloadList.observe(viewLifecycleOwner) {
                binding.splashProgress.visibility = View.GONE
                createList(it.list)
            }
            routeInfoList.observe(viewLifecycleOwner) {
                binding.splashProgress.visibility = View.GONE
                if (it.list.isNotEmpty()) {
                    binding.mainToolbar.toolbarIcons.visibility = View.VISIBLE
                    binding.mainToolbar.switchButton.visibility = View.VISIBLE
                } else {
                    binding.scanInput.setText("")
                    binding.mainToolbar.toolbarIcons.visibility = View.GONE
                    binding.mainToolbar.switchButton.visibility = View.GONE
                }
            }
            responseStateAdd.observe(viewLifecycleOwner) {
                binding.splashProgress.visibility = View.GONE
                if (!it.isError) {
                    counterResponse = counterResponse + 1
                    if (counterResponse == counterCalls) {
                        viewModel.getExpeditionFromRoute(159390)
                        counterResponse = 0
                        counterCalls = 0
                    }
                }

            }
        }
    }

    @RequiresApi(Build.VERSION_CODES.O)
    private fun setSwitchButon() {

        binding.mainToolbar.switchButton.setOnCheckedChangeListener { buttonView, isChecked ->
            binding.mainToolbar.switchButton.tooltipText = if (isChecked)
                getString(R.string.expeditionMarkLost) else
                getString(R.string.expeditionMarkFound)

            binding.splashProgress.visibility = View.VISIBLE

            if (isChecked) {
                binding.mainToolbar.toolbarTitle.text =
                    getString(R.string.loading)
            } else {
                getString(R.string.unloading)
            }
        }

    }

    private fun createList(list: List<ExpeditionInfoLoadUnload>) {
        myList = list as MutableList<ExpeditionInfoLoadUnload>
        adapter = ExpeditionLoadUnloadAdapter(list)
        binding.expeditionloadunloadRecyclerview.adapter = adapter
        binding.expeditionloadunloadRecyclerview.layoutManager =
            LinearLayoutManager(requireContext(), LinearLayoutManager.VERTICAL, false)
        binding.mainToolbar.toolbarSubtitle.text =
            getString(R.string.summaryCount, list.filter { it.code == state }.size, list.size)
        setListPosition(list)
    }

    private fun setListPosition(list: MutableList<ExpeditionInfoLoadUnload>) {
        for (i in list.indices) {
            if (list[i].code != state) {
                binding.expeditionloadunloadRecyclerview.scrollToPosition(i)
                break
            }
        }
    }

    private fun updateCount(list: MutableList<ExpeditionInfoLoadUnload>, state: String) {
        val countMarked = list.filter { it.code == state }.size
        getString(R.string.summaryCount, countMarked, list.size)
        if (countMarked == list.size) {
            setExpeditionsState(list, state)
        }
        adapter!!.notifyDataSetChanged()
    }
}