package es.verdnatura.presentation.view.feature.packaging.fragment

import android.content.Context
import android.os.Bundle
import android.view.View
import androidx.recyclerview.widget.LinearLayoutManager
import es.verdnatura.R
import es.verdnatura.databinding.FragmentPackagingBinding
import es.verdnatura.presentation.base.BaseFragment
import es.verdnatura.presentation.common.OnPasillerosItemClickListener
import es.verdnatura.presentation.view.feature.inventario.fragment.SearchSupplierModel
import es.verdnatura.presentation.view.feature.packaging.fragment.PackagingViewModel.PackagingViewModel
import es.verdnatura.presentation.view.feature.packaging.model.EntrySalix
import es.verdnatura.presentation.view.feature.packaging.model.Supplier
import es.verdnatura.presentation.view.feature.pasillero.model.PasillerosItemVO
import ir.mirrajabi.searchdialog.SimpleSearchDialogCompat


@Suppress("UNUSED_ANONYMOUS_PARAMETER")
class SupplierFragment(
    var entryPoint: String = ""

) : BaseFragment<FragmentPackagingBinding, PackagingViewModel>(
    PackagingViewModel::class
) {

    private var pasillerosItemClickListener: OnPasillerosItemClickListener? = null
    private var layoutManager: LinearLayoutManager? = null
    private var onBack = false
    private val suppliers = ArrayList<SearchSupplierModel>()


    companion object {
        fun newInstance(entryPoint: String) = SupplierFragment(entryPoint)
    }

    override fun getLayoutId(): Int = R.layout.fragment_packaging

    override fun onAttach(context: Context) {
        if (context is OnPasillerosItemClickListener) pasillerosItemClickListener = context
        super.onAttach(context)
    }

    override fun onCreate(savedInstanceState: Bundle?) {

        viewModel.getSuppliers(
        )

        super.onCreate(savedInstanceState)
    }

    override fun onPause() {
        onBack = true
        super.onPause()

    }

    override fun init() {

        binding.splashProgress.visibility = View.VISIBLE
        binding.filterSupplier.setOnClickListener {
            SimpleSearchDialogCompat(
                context,
                getString(R.string.suppliers),
                getString(R.string.escribirparteNombre),
                null,
                suppliers
            ) { baseSearchDialogCompat, nombre, position ->
                binding.filterSupplier.setText((nombre.getName()))
                binding.radiobuttonTypePackaging.visibility = View.VISIBLE
                saveData("SUPPLIERNAME", nombre.getName())
                saveData("SUPPLIERID", nombre.getId())
                baseSearchDialogCompat.dismiss()
            }.show()
        }

        ma.hideBottomNavigation(View.GONE)
        setToolBar()
        setEvents()

        val entry = EntrySalix(supplierFk = "591", travelFk = "186025", companyFk = "442")
        viewModel.entry_addSalix(entry)

        super.init()
    }

    private fun setToolBar() {
        binding.mainToolbar.toolbarTitle.text = entryPoint
    }

    private fun setEvents() {

        binding.mainToolbar.backButton.setOnClickListener {
            requireActivity().onBackPressed()
        }

        binding.radiobuttonTypePackaging.setOnCheckedChangeListener { buttonView, isChecked ->
            when (isChecked) {
                R.id.radioButtonRec -> {
                    saveData("ENTRYTYPE", "Rec")
                }
                R.id.radioButtonDev -> {

                    saveData("ENTRYTYPE", "Dev")
                }

            }
            if (!onBack) {
                ma.onPasillerosItemClickListener(
                    PasillerosItemVO(title = getString(R.string.titlePackagingCount)),
                    getString(R.string.titlePackagingCount)
                )

            }
            onBack = false

        }
    }


    override fun observeViewModel() {
        with(viewModel) {
            supplierList.observe(viewLifecycleOwner) {
                binding.splashProgress.visibility = View.GONE
                createSupplierList(it.list)
            }

            response.observe(viewLifecycleOwner) {
                if (it.isError) {
                    ma.messageWithSound(it.errorMessage, true, false)

                }
            }
        }
    }


    private fun createSupplierList(list: List<Supplier>) {
        suppliers.clear()
        list.forEach { supplier ->
            if (!supplier.isError) {
                try {
                    suppliers.add(SearchSupplierModel(supplier.name, supplier.id.toString()))
                } catch (e: Exception) {
                }
            }
        }
    }

}

